@@ -5,17 +5,202 @@ use strict;
 use IO::File;
 use IO::Select;
 use Carp;
+use UNIVERSAL ();
 
 use vars qw/ $VERSION /;
 local $^W = 1;
 
-$VERSION = '0.10';
+$VERSION = '0.11';
 
 my %month;
 @month{qw/ Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec /} = ( 0 .. 11 );
 
 my $DATE = qr/(?:Mon|Tue|Wed|Thu|Fri|Sat|Sun) (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) ([ \d]\d) (\d\d):(\d\d):(\d\d) .*(\d\d\d\d)/;
 
+sub new {
+    my $class = shift;
+    return bless {
+        fh     => new IO::File,
+        files  => [@_],
+        errors => 0,
+        buffer => [],
+    }, $class;
+}
+
+sub next {
+    my $log = shift;    # who needs $self?
+
+    # try to read a record (3 lines)
+    my $fh  = $log->fh();
+  READ:
+    {
+        my $read;
+
+      LINE:
+        while (<$fh>) {
+            $read++;
+
+            # should carp if doesn't get what's expected
+            # (From, then Subject, then Folder)
+
+            # From create a new Abstract
+            /^From (.+?) +($DATE)$/o && do {
+                push @{$log->{buffer}}, Log::Procmail::Abstract->new();
+
+                # assert: $read == 1;
+                $log->{buffer}[-1]->from($1);
+                $log->{buffer}[-1]->date($2);
+
+                # return ASAP
+                last READ if @{$log->{buffer}} > 1;
+                next LINE;
+            };
+
+            # assert: $read == 2;
+            /^ Subject: (.*)/i && do {
+                push @{$log->{buffer}}, Log::Procmail::Abstract->new()
+                    unless @{$log->{buffer}};
+                $log->{buffer}[0]->subject($1);
+                next LINE;
+            };
+
+            # procmail tabulates with tabs and spaces... :-(
+            # assert: $read == 3;
+            # Folder means the end of this record
+            /^  Folder: (.*?)\s+(\d+)$/ && do {
+                push @{$log->{buffer}}, Log::Procmail::Abstract->new()
+                  unless @{$log->{buffer}};
+
+                # assert: $read == 3;
+                $log->{buffer}[0]->folder($1);
+                $log->{buffer}[0]->size($2);
+                last READ;
+            };
+
+            # fall through: some error message
+            # shall we ignore it?
+            next LINE unless $log->{errors};
+
+            # or return it?
+            chomp;
+            push @{$log->{buffer}}, $_;
+            last LINE;
+        }
+
+        # in case we couldn't read the first line
+        if ( !$read or @{$log->{buffer}} == 0 ) {
+
+            # return ASAP
+            last READ if @{$log->{buffer}};
+
+            # go to next file
+            redo READ if $log->_open_next;
+
+            # unless it's the last one
+            return;
+        }
+    }
+
+    # we have an abstract
+    my $rec = shift @{$log->{buffer}};
+    if(UNIVERSAL::isa( $rec, 'Log::Procmail::Abstract')) {
+        # the folder field is required
+        goto READ unless defined $rec->folder;
+        $rec->{source} = $log->{source};
+    }
+
+    return $rec;
+}
+
+sub push {
+    my $log = shift;
+    push @{ $log->{files} }, @_;
+}
+
+sub errors {
+    my $self = shift;
+    return @_ ? $self->{errors} = shift() : $self->{errors};
+}
+
+sub fh {
+    my $log = shift;
+    $log->_open_next unless $log->{fh}->opened();
+    return $log->{fh};
+}
+
+sub select {
+    my $log = shift;
+    $log->fh(); # make sure the file is correctly opened and select is updated
+    return $log->{select};
+}
+
+# *internal method*
+# opens a file or replace the old filehandle by the new one
+# push() can therefore accept refs to typeglobs, IO::Handle, or filenames
+sub _open_next {
+    my ( $log ) = @_;
+    my $file;
+
+    if ( @{ $log->{files} } ) {
+        $file = shift @{ $log->{files} };
+    } else { return 0 };
+
+    if ( ref $file eq 'GLOB' ) {
+        $log->{fh} = *$file{IO};
+        carp "Closed filehandle $log->{fh}" unless $log->{fh}->opened;
+    }
+    elsif ( ref $file && $file->isa('IO::Handle') ) {
+        $log->{fh} = $file;
+    }
+    else {
+        $log->{fh}->open($file) or carp "Can't open $file: $!";
+    }
+    $log->{source} = $file;
+    $log->{select} = ( grep $^O eq $_, qw(MSWin32 NetWare dos VMS riscos beos) )
+                   ? undef
+                   : IO::Select->new( $log->{fh} );
+    1;
+}
+
+sub DESTROY {
+    my $self = shift;
+    if ( $self->{fh} && $self->{fh}->opened ) { $self->{fh}->close }
+}
+
+#
+# a small class for the abstracts themselves
+#
+package Log::Procmail::Abstract;
+
+use Carp;
+
+sub new {
+    my $class = shift;
+    return bless {@_}, $class;
+}
+
+for my $attr (qw( from date subject size folder source ) ) {
+    no strict 'refs';
+    *$attr = sub {
+        my $self = shift;
+        @_ ? $self->{$attr} = shift: $self->{$attr};
+    }
+}
+
+sub ymd {
+    my $self = shift;
+    croak("Log::Procmail::Abstract::ymd cannot be used to set the date")
+      if @_;
+    return undef unless defined $self->{date};
+    $self->{date} =~ /^$DATE$/o;
+    return undef unless $1;
+    return sprintf( "%04d%02d%02d$3$4$5", $6, $month{$1} + 1, $2 );
+}
+
+1;
+
+__END__
+
 =head1 NAME
 
 Log::Procmail - Perl extension for reading procmail logfiles.
@@ -55,18 +240,6 @@ When $log reaches the end of the file "log", it doesn't close the file.
 So, after B<procmail> processes some incoming mail, the next call to next()
 will return the new records.
 
-=cut
-
-sub new {
-    my $class = shift;
-    return bless {
-        fh     => new IO::File,
-        files  => [@_],
-        errors => 0,
-        buffer => [],
-    }, $class;
-}
-
 =item $rec = $log->next
 
 Return a Log::Procmail::Abstract object that represent an entry in the log
@@ -107,91 +280,6 @@ Log::Procmail::Abstract object. Here is an example:
         # normal log processing
     }
 
-=cut
-
-sub next {
-    my $log = shift;    # who needs $self?
-
-    # try to read a record (3 lines)
-    my $fh  = $log->fh;
-  READ:
-    {
-        my $read;
-        while (<$fh>) {
-            $read++;
-
-            # should carp if doesn't get what's expected
-            # (From, then Subject, then Folder)
-
-            # From create a new Abstract
-            /^From (.+?) +($DATE)$/o && do {
-                push @{$log->{buffer}}, Log::Procmail::Abstract->new;
-
-                # assert: $read == 1;
-                $log->{buffer}[-1]->from($1);
-                $log->{buffer}[-1]->date($2);
-
-                # return ASAP
-                last READ if @{$log->{buffer}} > 1;
-                next;
-            };
-
-            # assert: $read == 2;
-            /^ Subject: (.*)/i && do {
-                push @{$log->{buffer}}, Log::Procmail::Abstract->new
-                    unless @{$log->{buffer}};
-                $log->{buffer}[0]->subject($1);
-                next;
-            };
-
-            # procmail tabulates with tabs and spaces... :-(
-            # assert: $read == 3;
-            # Folder means the end of this record
-            /^  Folder: (.*?)\s+(\d+)$/ && do {
-                push @{$log->{buffer}}, Log::Procmail::Abstract->new
-                  unless @{$log->{buffer}};
-
-                # assert: $read == 3;
-                $log->{buffer}[0]->folder($1);
-                $log->{buffer}[0]->size($2);
-                last READ;
-            };
-
-            # fall through: some error message
-            # shall we ignore it?
-            next unless $log->{errors};
-
-            # or return it?
-            chomp;
-            push @{$log->{buffer}}, $_;
-            last;
-        }
-
-        # in case we couldn't read the first line
-        if ( !$read or @{$log->{buffer}} == 0 ) {
-
-            # return ASAP
-            last READ if @{$log->{buffer}};
-
-            # go to next file
-            redo READ if $log->_open_next;
-
-            # unless it's the last one
-            return;
-        }
-    }
-
-    # we have an abstract
-    my $rec = shift @{$log->{buffer}};
-    if($rec->isa( 'Log::Procmail::Abstract')) {
-        # the folder field is required
-        goto READ unless defined $rec->folder;
-        $rec->{source} = $log->{source};
-    }
-
-    return $rec
-}
-
 =item $log->push( $file [, $file2 ...] );
 
 Push one or more files on top of the list of log files to examine.
@@ -199,13 +287,6 @@ When Log::Procmail runs out of abstracts to return (i.e. it reaches the
 end of the file), it transparently opens the next file (if there is one)
 and keeps returning new abstracts.
 
-=cut
-
-sub push {
-    my $log = shift;
-    push @{ $log->{files} }, @_;
-}
-
 =item $log->errors( [bool] );
 
 Set or get the error flag. If set, when the next() method will return
@@ -214,26 +295,11 @@ it is a simple string, not a Log::Procmail::Abstract object.
 
 Default is to return no error.
 
-=cut
-
-sub errors {
-    my $self = shift;
-    @_ ? $self->{errors} = shift: $self->{errors};
-}
-
 =item $fh = $log->fh()
 
 Returns the currently opened filehandle, from which the next call to
 C<next()> will try to read a record.
 
-=cut
-
-sub fh {
-    my $log = shift;
-    $log->_open_next unless $log->{fh}->opened;
-    $log->{fh};
-}
-
 =item $select = $log->select()
 
 Return a IO::Select object that watches the currently opened filehandle.
@@ -246,47 +312,6 @@ and C<beos>:> on those systems, C<select()> returns C<undef>.
 (Check F<ext/IO/t/io_sel.t> in the Perl sources for details.
 Hint: look for the message I<4-arg select is only valid on sockets>.)
 
-=cut
-
-sub select {
-    my $log = shift;
-    $log->fh; # make sure the file is correctly opened and select is updated
-    $log->{select};
-}
-
-# *internal method*
-# opens a file or replace the old filehandle by the new one
-# push() can therefore accept refs to typeglobs, IO::Handle, or filenames
-sub _open_next {
-    my ( $log ) = @_;
-    my $file;
-
-    if ( @{ $log->{files} } ) {
-        $file = shift @{ $log->{files} };
-    } else { return 0 };
-
-    if ( ref $file eq 'GLOB' ) {
-        $log->{fh} = *$file{IO};
-        carp "Closed filehandle $log->{fh}" unless $log->{fh}->opened;
-    }
-    elsif ( ref $file && $file->isa('IO::Handle') ) {
-        $log->{fh} = $file;
-    }
-    else {
-        $log->{fh}->open($file) or carp "Can't open $file: $!";
-    }
-    $log->{source} = $file;
-    $log->{select} = ( grep $^O eq $_, qw(MSWin32 NetWare dos VMS riscos beos) )
-                   ? undef
-                   : IO::Select->new( $log->{fh} );
-    1;
-}
-
-sub DESTROY {
-    my $self = shift;
-    if ( $self->{fh} && $self->{fh}->opened ) { $self->{fh}->close }
-}
-
 =back
 
 =head2 Log::Procmail::Abstract
@@ -298,17 +323,6 @@ folder() and size() methods.
 
 Log::Procmail::next() returns a Log::Procmail::Abstract object.
 
-=cut
-
-package Log::Procmail::Abstract;
-
-use Carp;
-
-sub new {
-    my $class = shift;
-    return bless {@_}, $class;
-}
-
 =over 4
 
 =item Log::Procmail::Abstract accessors
@@ -324,33 +338,11 @@ otherwise.
 The source() accessor returns the name of the log file or the string
 representation of the handle, if a filehandle was given.
 
-=cut
-
-for my $attr (qw( from date subject size folder source ) ) {
-    no strict 'refs';
-    *$attr = sub {
-        my $self = shift;
-        @_ ? $self->{$attr} = shift: $self->{$attr};
-    }
-}
-
 =item $rec->ymd()
 
 Return the date in the form C<yyyymmmddhhmmss> where each field is what
 you think it is. C<;-)> This method is read-only.
 
-=cut
-
-sub ymd {
-    my $self = shift;
-    croak("Log::Procmail::Abstract::ymd cannot be used to set the date")
-      if @_;
-    return undef unless defined $self->{date};
-    $self->{date} =~ /^$DATE$/o;
-    return undef unless $1;
-    return sprintf( "%04d%02d%02d$3$4$5", $6, $month{$1} + 1, $2 );
-}
-
 =back
 
 =head1 EXAMPLES
@@ -406,12 +398,14 @@ when the file is rotated.
 
 Please report all bugs through the rt.cpan.org interface:
 
-http://rt.cpan.org/NoAuth/Bugs.html?Dist=Log-Procmail
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Log-Procmail>
 
 =head1 AUTHOR
 
 Philippe "BooK" Bruhat <book@cpan.org>.
 
+=head1 ACKNOWLEDGMENTS
+
 Thanks to Briac "Oeufmayo" Pilpr and David "Sniper" Rigaudiere for early
 comments on irc. Thanks to Olivier "rs" Poitrey for giving me his huge
 procmail log file (51 Mb spanning over a two-year period) and for probably
@@ -437,4 +431,3 @@ perl(1), procmail(1).
 
 =cut
 
-1;
